@@ -3,7 +3,7 @@ package Hades;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 use Module::Generate;
 use Switch::Again qw/switch/;
 
@@ -120,8 +120,10 @@ sub build_new {
 	my $accessors = q|(|;
 	map {
 		$accessors .= qq|$_ => {|;
-		$accessors .= qq|required=>1,| if $meta->{$_}->{required};
-		$accessors .= qq|default=>$meta->{$_}->{default},| if $meta->{$_}->{default};
+		$accessors .= qq|required => 1,| if $meta->{$_}->{required};
+		$accessors .= qq|default => $meta->{$_}->{default},| if $meta->{$_}->{default};
+		$accessors .= qq|builder => sub { my (\$self, \$value) = \@_;| . $self->build_builder($_, '$value', $meta->{$_}->{builder}) . qq|return \$value;}| 
+			if $meta->{$_}->{builder};
 		$accessors .= qq|},|;
 	} grep { $meta->{$_}->{meta} eq 'ACCESSOR' } keys %{$meta};
 	$accessors .= q|)|;
@@ -131,7 +133,13 @@ sub build_new {
 		$new;
 		my \%accessors = $accessors;
 		for my \$accessor ( keys \%accessors ) {
-			my \$value = \$self->\$accessor(defined \$args{\$accessor} ? \$args{\$accessor} : \$accessors{\$accessor}->{default});
+			my \$param = defined \$args{\$accessor} ? \$args{\$accessor} : \$accessors{\$accessor}->{default};
+			my \$value = \$self->\$accessor(
+				\$accessors{\$accessor}->{builder} ? \$accessors{\$accessor}->{builder}->(
+					\$self,
+					\$param
+				) : \$param
+			);
 			unless (!\$accessors{\$accessor}->{required} \|\| defined \$value) {
 				die "\$accessor accessor is required";
 			}
@@ -161,7 +169,7 @@ sub build_sub_no_arguments {
 	my ($self, $mg, $token, $meta) = @_;
 	my $name = shift @{$token};
 	$name =~ m/^(begin|unitcheck|check|init|end|new)$/
-		? $mg->$name(join ' ', @{$token})
+		? $mg->$name('{' . join( ' ', @{$token}) . '}')
 		: $mg->sub($name)->code($self->build_code($mg, $name, $self->build_sub_code($name, '', '', join ' ', @{$token})))
 		->pod(qq|call $name method. Expects no params.|)->example(qq|\$obj->$name()|);
 	return $meta;
@@ -201,21 +209,26 @@ sub build_sub_or_accessor_attributes {
 			my $value = shift;
 			push @{$meta->{$name}->{caught}}, $value;
 		},
-		qr/^(\:around|\:ar)/ => sub {
+		qr/^(\:around|\:ar)$/ => sub {
 			$meta->{$name}->{meta} = 'MODIFY';
 			$token->[-1] =~ s/^\{(.*)\}$/$1/sg;
 			$meta->{$name}->{around} = pop @{$token};
 		},
-		qr/^(\:after|\:a)/ => sub {
+		qr/^(\:after|\:a)$/ => sub {
 			$meta->{$name}->{meta} = 'MODIFY';
 			$token->[-1] =~ s/^\{(.*)\}$/$1/sg;
 			$meta->{$name}->{after} = pop @{$token};
 		},
-		qr/^(\:before|\:b)/ => sub {
+		qr/^(\:before|\:b)$/ => sub {
 			$meta->{$name}->{meta} = 'MODIFY';
 			$token->[-1] =~ s/^\{(.*)\}$/$1/sg;
 			$meta->{$name}->{before} = pop @{$token};
 		},
+		qr/^(:builder|:bdr)/ => sub {
+			my $value = shift;
+			$value =~ s/(\:bd|\:build)\((.*)\)$/$2/sg;
+			$meta->{$name}->{builder} = $2 ? $value : 1;
+		},
 		qr/^(\:clearer|\:c)$/ => sub {
 			$meta->{$name}->{clearer} = 1;
 		},
@@ -233,7 +246,7 @@ sub build_sub_or_accessor_attributes {
 			my $value = shift;
 			$value =~ s/.*\((.*)\)/$1/sg;
 			$value = '"' . $value . '"'
-				if $value !~ m/^(\{|\[|\"|\'|q)|(\d+)/;
+				if $value !~ m/^(\{|\[|\"|\'|\$|\£|q)|(\d+)/;
 			$meta->{$name}->{default} =  $value;
 			if ($meta->{$name}->{params_map}) {
 				$meta->{$name}->{params_map}->{
@@ -483,6 +496,17 @@ sub build_predicate_code {
 	}|;
 }
 
+sub build_builder {
+	my ($self, $name, $param, $code) = @_;
+	if (defined $code) {
+		$code = "_build_$name" if $code =~ m/^1$/;
+		return $code =~ m/^\w+$/
+			? qq|$param = \$self->$code($param);|
+			: $code
+	}
+	return q||;
+}
+
 sub build_coerce {
 	my ($self, $name, $param, $code) = @_;
 	return defined $code ? $code =~ m/^\w+$/
@@ -1108,8 +1132,8 @@ sub build_tests {
 		$meta->{private} ? do {
 			push @tests, ['eval', qq|\$obj->$name|, 'private method|private attribute'];
 		} : do {
-			push @tests, ['is', qq|\$obj->$name|, 'undef'] if !$meta->{required} && !$meta->{default};
-			my (@test_cases) = $self->build_test_data($meta->{type}->[0] || 'Any', $name);
+			push @tests, ['is', qq|\$obj->$name|, 'undef'] if !$meta->{builder} && !$meta->{required} && !$meta->{default};
+			my (@test_cases) = $self->build_test_data($meta->{type}->[0] || 'Any', $name, $meta->{required} || $meta->{builder});
 			if (scalar @test_cases > 1) {
 				my $valid = shift @test_cases;
 				push @tests, ['deep', qq|\$obj->$name($valid)|, $valid];
@@ -1172,7 +1196,7 @@ Hades - Less is more, more is less!
 
 =head1 VERSION
 
-Version 0.15
+Version 0.16
 
 =cut
 
@@ -1586,6 +1610,15 @@ Add type checking to the accessor.
 
 =cut
 
+=head3 :builder | :bdr
+
+Takes a coderef which is meant to build the attributes value.
+
+	dokimi :bdr
+	dokimes :builder($value = $value->[0] if ref($value) || "" eq "ARRAY";)
+
+=cut
+
 =head3 :test | :z
 
 Add tests associated to the accessor.
@@ -1890,7 +1923,7 @@ Used in conjunction with Dict and Tuple to specify slots that are optional and m
 
 =head2 Macros
 
-Hades has a concept of macros that allow you to write re-usable code. see L<https://metacpan.org/source/LNATION/Hades-0.15/macro-fh.hades> for an example of how to extend via macros.
+Hades has a concept of macros that allow you to write re-usable code. see L<https://metacpan.org/source/LNATION/Hades-0.16/macro-fh.hades> for an example of how to extend via macros.
 
 	macro {
 		FH [ macro => [qw/read_file write_file/], alias => { read_file => [qw/rf/], write_file => [qw/wf/] } ]
